#include "cdk.h"
#include <limits.h>

/*
 * $Author: glovem $
 * $Date: 1997/04/25 12:52:33 $
 * $Revision: 1.64 $
 */

/*
 * Declare file local prototypes.
 */
void CDKTemplateCallBack (CDKTEMPLATE *template, chtype input);
void drawCDKTemplateField (CDKTEMPLATE *template);
void adjustCDKTemplateCursor (CDKTEMPLATE *template, int direction);

/*
 * Declare file local variables.
 */
extern char *GPasteBuffer;

/*
 * This creates a template widget.
 */
CDKTEMPLATE *newCDKTemplate (CDKSCREEN *cdkscreen, int xplace, int yplace, char *title, char *label, char *plate, char *overlay, boolean box, boolean shadow)
{
   /* Set up some variables */
   CDKTEMPLATE *template	= (CDKTEMPLATE *)malloc (sizeof (CDKTEMPLATE));
   chtype *holder		= (chtype *)NULL;
   int parentWidth		= WIN_WIDTH (cdkscreen->window);
   int parentHeight		= WIN_HEIGHT (cdkscreen->window);
   int boxWidth			= 0;
   int boxHeight		= 3;
   int maxWidth			= INT_MIN;
   int xpos			= xplace;
   int ypos			= yplace;
   int fieldWidth		= 0;
   int horizontalAdjust		= 0;
   int junk			= 0;
   char *temp[256];
   int x, len, junk2;
   
   /* Make sure the plate is not NULL. */
   if (plate == (char *)NULL)
   {
      free (template);
      return (CDKTEMPLATE *)NULL;
   }
   fieldWidth = (int)strlen (plate) + 2;

   /* Set some basic values of the template field. */
   template->label	= (chtype *)NULL;
   template->labelLen	= 0;
   template->labelWin	= (WINDOW *)NULL;
   template->titleLines	= 0;

   /* Translate the char * label to a chtype * */
   if (label != (char *)NULL)
   {
      template->label	= char2Chtype (label, &template->labelLen, &junk);
   }

   /* Translate the char * overlay to a chtype * */
   if (overlay != (char *)NULL)
   {
      template->overlay = char2Chtype (overlay, &template->overlayLen, &junk);
      template->fieldAttr = template->overlay[0] & A_ATTRIBUTES;
   }
   else
   {
      template->overlay		= (chtype *)NULL;
      template->overlayLen	= 0;
      template->fieldAttr	= A_NORMAL;
   }

   /* Set the box width. */
   boxWidth = fieldWidth+ template->labelLen + 2;
   
   /* Translate the char * items to chtype * */
   if (title != (char *)NULL)
   {
      /* We need to split the title on \n. */
      template->titleLines = splitString (title, temp, '\n');
 
      /* We need to determine the widest title line. */
      for (x=0; x < template->titleLines; x++)
      {
         holder = char2Chtype (temp[x], &len, &junk2);
         maxWidth = MAXIMUM (maxWidth, len);
         freeChtype (holder);
      }
 
      /*
       * If one of the title lines is wider than the field and the label,
       * the box width will expand to accomodate.
       */
       if (maxWidth > boxWidth)
       {
          horizontalAdjust = (int)((maxWidth - boxWidth) / 2) + 1;
          boxWidth = maxWidth + 2;
       }

      /* For each line in the title, convert from char * to chtype * */
      for (x=0; x < template->titleLines; x++)
      {
         template->title[x]	= char2Chtype (temp[x], &template->titleLen[x], &template->titlePos[x]);
         template->titlePos[x]	= justifyString (boxWidth, template->titleLen[x], template->titlePos[x]);
         freeChar (temp[x]);
      }
   }
   else
   {
      /* No title? Set the required variables. */
      template->titleLines = 0;
   }
   boxHeight += template->titleLines;

  /*
   * Make sure we didn't extend beyond the dimensions of the window.
   */
   boxWidth = (boxWidth > parentWidth ? parentWidth : boxWidth);
   boxHeight = (boxHeight > parentHeight ? parentHeight : boxHeight);
   fieldWidth = (fieldWidth > (boxWidth - template->labelLen - 2) ? (boxWidth - template->labelLen - 2) : fieldWidth);

   /* Rejustify the x and y positions if we need to. */
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Make the template window */
   template->win = newwin (boxHeight, boxWidth, ypos, xpos);
   
   /* Is the template window NULL?? */
   if (template->win == (WINDOW *)NULL)
   {
      /* Clean up any used memory. */
      freeChtype (template->label);
      freeChtype (template->overlay);
      free (template);

      /* Return a NULL pointer. */
      return ((CDKTEMPLATE *)NULL);
   }
   keypad (template->win, TRUE);

   /* Make the label window. */
   if (template->label != (chtype *)NULL)
   {
      template->labelWin = subwin (template->win, 1, 
					template->labelLen+2,
					ypos+template->titleLines+1,
					xpos+horizontalAdjust+1);
   }

   /* Make the field window. */
   template->fieldWin = subwin (template->win, 1, fieldWidth,
				ypos+template->titleLines+1,
				xpos+template->labelLen+horizontalAdjust+2);
   keypad (template->fieldWin, TRUE);

   /* Set up the info field. */
   template->plateLen = (int)strlen(plate);
   template->info = (char *)malloc (sizeof(char) * (template->plateLen+2));
   cleanChar (template->info, template->plateLen+1, '\0');

   /* Set up the rest of the structure  */
   template->parent			= cdkscreen->window;
   template->shadowWin			= (WINDOW *)NULL;
   template->plate			= copyChar (plate);
   template->fieldWidth			= fieldWidth;
   template->boxHeight			= boxHeight;
   template->boxWidth			= boxWidth;
   template->platePos			= 0;
   template->screenPos			= 0;
   template->infoPos			= 0;
   template->exitType			= vNEVER_ACTIVATED;
   template->min			= 0;
   template->box			= box;
   template->shadow			= shadow;
   template->callbackfn			= (void *)&CDKTemplateCallBack;
   template->preProcessFunction		= (PROCESSFN)NULL;
   template->preProcessData		= (void *)NULL;
   template->postProcessFunction	= (PROCESSFN)NULL;
   template->postProcessData		= (void *)NULL;
   template->ULChar			= ACS_ULCORNER;
   template->URChar			= ACS_URCORNER;
   template->LLChar			= ACS_LLCORNER;
   template->LRChar			= ACS_LRCORNER;
   template->HChar			= ACS_HLINE;
   template->VChar			= ACS_VLINE;
   template->BoxAttrib			= A_NORMAL;

   /* Do we need to create a shaodw??? */
   if (shadow)
   {
      template->shadowWin = newwin (boxHeight, boxWidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings. */
   cleanCDKObjectBindings (vTEMPLATE, template);

   /* Register this baby.                */
   registerCDKObject (cdkscreen, vTEMPLATE, template);

   /* Return the pointer to the structure */
   return template;
}

/*
 * This actually manages the template widget...
 */
char *activateCDKTemplate (CDKTEMPLATE *template, chtype *actions)
{
   /* Declare local variables. */
   chtype input	= (chtype)NULL;
   char *ret	= (char *)NULL;

   /* Draw the object. */
   drawCDKTemplate (template, template->box);

   /* Check if actions is NULL. */
   if (actions == (chtype *)NULL)
   {
      for (;;)
      {
         /* Get the input. */
         input = wgetch (template->win);
 
         /* Inject the character into the widget. */
         ret = injectCDKTemplate (template, input);
         if (template->exitType != vEARLY_EXIT)
         {
            return ret;
         }
      }
   }
   else
   {
      int length = chlen (actions);
      int x = 0;
 
      /* Inject each character one at a time. */
      for (x=0; x < length; x++)
      {
         ret = injectCDKTemplate (template, actions[x]);
         if (template->exitType != vEARLY_EXIT)
         {
            return ret;
         }
      }
   }

   /* Set the exit type and return. */
   template->exitType = vEARLY_EXIT;
   return ret;
}

/*
 * This injects a character into the widget.
 */
char *injectCDKTemplate (CDKTEMPLATE *template, chtype input)
{
   /* Declare some local variables. */
   int ppReturn = 1;
   int length, x;

   /* Set the exit type and return. */
   template->exitType = vEARLY_EXIT;

   /* Move the cursor. */
   drawCDKTemplateField (template);

   /* Should we call a post-process? */
   if (template->postProcessFunction != (PROCESSFN)NULL)
   {
      ((PROCESSFN)(template->postProcessFunction)) (vTEMPLATE, template, template->preProcessData, input);
   }

   /* Should we continue? */
   if (ppReturn != 0)
   {
      /* Check a predefined binding...           */
      if (! checkCDKObjectBind (vTEMPLATE, template, input))
      {
         switch (input)
         {
            case KEY_LEFT : case KEY_RIGHT : case KEY_UP : case KEY_DOWN :
                 Beep();
                 break;

            case CDK_ERASE :
                 if ((int)strlen(template->info) != 0)
                 {
                    cleanChar (template->info, template->plateLen+1, '\0');
                    template->screenPos = 0;
                    template->infoPos = 0;
                    template->platePos = 0;
                    drawCDKTemplateField (template);
                 }
                 break;

            case CDK_CUT:
                 if ((int)strlen(template->info) != 0)
                 {
	            freeChar (GPasteBuffer);
                    GPasteBuffer = copyChar (template->info);
                    cleanChar (template->info, template->plateLen+1, '\0');
                    template->screenPos = 0;
                    template->infoPos = 0;
                    template->platePos = 0;
                    drawCDKTemplateField (template);
                 }
                 else
                 {
                    Beep();
                 }
                 break;

            case CDK_COPY:
                 if ((int)strlen(template->info) != 0)
                 {
	            freeChar (GPasteBuffer);
                    GPasteBuffer = copyChar (template->info);
                 }
                 else
                 {
                    Beep();
                 }
                 break;

            case CDK_PASTE:
                 if (GPasteBuffer != (char *)NULL)
                 {
                    /* Clean the informatiob from the field. */
                    cleanChar (template->info, template->plateLen+1, '\0');
                    template->screenPos = 0;
                    template->infoPos = 0;
                    template->platePos = 0;

                    /* Start inserting each character one at a time. */
                    length = (int)strlen (GPasteBuffer);
                    for (x=0; x < length; x++)
                    {
                       ((TEMPLATECB)template->callbackfn)(template, GPasteBuffer[x]);
                    }
                    drawCDKTemplateField (template);
                 }
                 else
                 {
                    Beep();
                 }
                 break;

            case KEY_RETURN : case KEY_TAB : case KEY_ENTER :
                 if (strlen(template->info) < template->min)
                 {
                    Beep();
                 }
                 else
                 {
                    template->exitType = vNORMAL;
                    return template->info;
                 }
                 break;
   
            case KEY_ESC :
                 template->exitType = vESCAPE_HIT;
                 return (char *)NULL;
                 break;
   
            case CDK_REFRESH :
                 eraseCDKScreen (template->screen);
                 refreshCDKScreen (template->screen);
                 break;
            
            default :
                 ((TEMPLATECB)template->callbackfn)(template, input);
                 break;
         }
      }

      /* Should we call a post-process? */
      if (template->postProcessFunction != (PROCESSFN)NULL)
      {
         ((PROCESSFN)(template->postProcessFunction)) (vTEMPLATE, template, template->postProcessData, input);
      }
   }

   /* Set the exit type and return. */
   template->exitType = vEARLY_EXIT;
   return (char *)NULL;
}

/*
 * This is the standard callback proc for the template.
 */
void CDKTemplateCallBack (CDKTEMPLATE *template, chtype input)
{
   chtype fieldColor;

   /* Check the character input. */
   if (input == DELETE || input == KEY_BACKSPACE ||
	input == '' || input == KEY_DC)
   {
      /* Make sure we don't go out of bounds. */
      if (template->platePos == 0 || template->info[0] == '\0')
      {
         Beep();
         return;
      }

      /* Remove it and redisplay.... */
      template->info[--template->infoPos] = '\0';
      template->platePos--;
      template->screenPos--;

      /* Move the cursor */
      fieldColor = template->overlay[template->screenPos] & A_ATTRIBUTES;
      adjustCDKTemplateCursor (template, -1);
      mvwaddch (template->fieldWin, 0, template->screenPos,
		template->overlay[template->platePos] | fieldColor);
      adjustCDKTemplateCursor (template, -1);
   }
   else
   {
      /* To be safe, we should cast the chtype to a char */
      char newchar = (char)input;

      /* Make sure we don't go out of bounds. */
      if (template->platePos >= template->fieldWidth)
      {
         Beep();
         return;
      }

      /* We should check to see if what they typed in matches */
      /* what the plate states. */
      if (isdigit(newchar) && ( \
		template->plate[template->platePos] == 'A' || \
		template->plate[template->platePos] == 'C' || \
		template->plate[template->platePos] == 'c'))
      {
         Beep();
         return;
      }
      if (!isdigit(newchar) && template->plate[template->platePos] == '#')
      {
         Beep();
         return;
      }

      /* Do we need to convert the case??? */
      if (template->plate[template->platePos] == 'C' || \
		template->plate[template->platePos] == 'X')
      {
         newchar = toupper ((char)input);
      }
      else if (template->plate[template->platePos] == 'c' || \
		template->plate[template->platePos] == 'x')
      {
         newchar = tolower ((char)input);
      }

      /* Add it and redisplay.... */
      fieldColor = template->overlay[template->screenPos] & A_ATTRIBUTES;
      template->info[template->infoPos++] = newchar;
      template->info[template->infoPos] = '\0';
      mvwaddch (template->fieldWin, 0, template->screenPos,
			newchar | fieldColor);
      template->platePos++;
      template->screenPos++;

      /* Move the cursor */
      adjustCDKTemplateCursor (template, +1);
   }
}

/*
 * This takes the overlay and the info, and mixes the two, for a
 * 'mixed' character string...
 */
char *mixCDKTemplate (CDKTEMPLATE *template)
{
   /* Reconstruct the info. */
   char *mixedString	= (char *)NULL;
   int platePos		= 0;
   int infoPos		= 0;
   
   /* Check if the string is NULL. */
   if (template->info == (char *)NULL ||
	(int)strlen(template->info) == 0)
   {
      return (char *)NULL;
   }

   /* Make some room for the mixed string. */
   mixedString = (char *) malloc ((sizeof (char) * template->plateLen) + 3);
   cleanChar (mixedString, template->plateLen+3, '\0');

   /* Start copying from the plate to the destination string. */
   while (platePos < template->plateLen)
   {
      if (template->plate[platePos] == '#' || \
		template->plate[platePos] == 'A' || \
		template->plate[platePos] == 'C' || \
		template->plate[platePos] == 'c' || \
		template->plate[platePos] == 'M' || \
		template->plate[platePos] == 'X' || \
		template->plate[platePos] == 'x')
      {
         mixedString[platePos]	= template->info[infoPos++];
      }
      else
      {
         mixedString[platePos]	= template->plate[platePos];
      }
      platePos++;
   }

   /* Return the new string. */
   return mixedString;
}

/*
 * This takes a mixed string and returns a non-mixed string.
 */
char *unmixCDKTemplate (CDKTEMPLATE *template, char *info)
{
   /* Reconstruct the info. */
   char *unmixedString	= (char *)NULL;
   int infolen		= (int)strlen (info);
   int x		= 0;
   int pos		= 0;

   /* Create a char * pointer. */
   unmixedString	= (char *)malloc (sizeof(char) * (infolen+2));
   cleanChar (unmixedString, infolen+2, '\0');

   /* Start copying. */
   while (pos < infolen)
   {
      if (template->plate[pos] == '#' || \
		template->plate[pos] == 'A' || \
		template->plate[pos] == 'C' || \
		template->plate[pos] == 'c' || \
		template->plate[pos] == 'M' || \
		template->plate[pos] == 'X' || \
		template->plate[pos] == 'x')
      {
         unmixedString[x++] = info[pos++];
      }
      else
      {
         pos++;
      }
   }

   /* Return the new string. */
   return unmixedString;
}

/*
 * This moves the template field to the given location.
 */
void moveCDKTemplate (CDKTEMPLATE *template, int xplace, int yplace, boolean relative, boolean refresh)
{
   /* Declare local variables. */
   int currentX = template->win->_begx;
   int currentY = template->win->_begy;
   int xpos	= xplace;
   int ypos	= yplace;
   int xdiff	= 0;
   int ydiff	= 0;

   /*
    * If this is a relative move, then we will adjust where we want
    * to move to.
     */
   if (relative)
   {
      xpos = template->win->_begx + xplace;
      ypos = template->win->_begy + yplace;
   }

   /* Adjust the window if we need to. */
   alignxy (template->screen->window, &xpos, &ypos, template->boxWidth, template->boxHeight);

   /* Get the difference. */
   xdiff = currentX - xpos;
   ydiff = currentY - ypos;

   /* Move the window to the new location. */
   template->win->_begx = xpos;
   template->win->_begy = ypos;
   if (template->labelWin != (WINDOW *)NULL)
   {
      template->labelWin->_begx -= xdiff;
      template->labelWin->_begy -= ydiff;
   }
   template->fieldWin->_begx -= xdiff;
   template->fieldWin->_begy -= ydiff;

   /* If there is a shadow box we have to move it too. */
   if (template->shadowWin != (WINDOW *)NULL)
   {
      template->shadowWin->_begx -= xdiff;
      template->shadowWin->_begy -= ydiff;
   }

   /* Touch the windows so they 'move'. */
   touchwin (template->screen->window);
   wrefresh (template->screen->window);

   /* Redraw the window, if they asked for it. */
   if (refresh)
   {
      drawCDKTemplate (template, template->box);
   }
}

/*
 * This allows the user to use the cursor keys to adjust the
 * position of the widget.
 */
void positionCDKTemplate (CDKTEMPLATE *template)
{
   /* Declare some variables. */
   int origX	= template->win->_begx;
   int origY	= template->win->_begy;
   chtype key	= (chtype)NULL;

   /* Let them move the widget around until they hit return. */
   while ((key != KEY_RETURN) && (key != KEY_ENTER))
   {
      key = wgetch (template->win);
      if (key == KEY_UP || key == '8')
      {
         if (template->win->_begy > 0)
         {
            moveCDKTemplate (template, 0, -1, TRUE, TRUE);
         }
         else
         {
            Beep();
         }
      }
      else if (key == KEY_DOWN || key == '2')
      {
         if (template->win->_begy+template->win->_maxy < template->screen->window->_maxy-1)
         {
            moveCDKTemplate (template, 0, 1, TRUE, TRUE);
         }
         else
         {
            Beep();
         }
      }
      else if (key == KEY_LEFT || key == '4')
      {
         if (template->win->_begx > 0)
         {
            moveCDKTemplate (template, -1, 0, TRUE, TRUE);
         }
         else
         {
            Beep();
         }
      }
      else if (key == KEY_RIGHT || key == '6')
      {
         if (template->win->_begx+template->win->_maxx < template->screen->window->_maxx-1)
         {
            moveCDKTemplate (template, 1, 0, TRUE, TRUE);
         }
         else
         {
            Beep();
         }
      }
      else if (key == '7')
      {
         if (template->win->_begy > 0 && template->win->_begx > 0)
         {
            moveCDKTemplate (template, -1, -1, TRUE, TRUE);
         }
         else
         {
            Beep();
         }
      }
      else if (key == '9')
      {
         if (template->win->_begx+template->win->_maxx < template->screen->window->_maxx-1 &&
		template->win->_begy > 0)
         {
            moveCDKTemplate (template, 1, -1, TRUE, TRUE);
         }
         else
         {
            Beep();
         }
      }
      else if (key == '1')
      {
         if (template->win->_begx > 0 && template->win->_begx+template->win->_maxx < template->screen->window->_maxx-1)
         {
            moveCDKTemplate (template, -1, 1, TRUE, TRUE);
         }
         else
         {
            Beep();
         }
      }
      else if (key == '3')
      {
         if (template->win->_begx+template->win->_maxx < template->screen->window->_maxx-1 &&
		template->win->_begy+template->win->_maxy < template->screen->window->_maxy-1)
         {
            moveCDKTemplate (template, 1, 1, TRUE, TRUE);
         }
         else
         {
            Beep();
         }
      }
      else if (key == '5')
      {
         moveCDKTemplate (template, CENTER, CENTER, FALSE, TRUE);
      }
      else if (key == 't')
      {
         moveCDKTemplate (template, template->win->_begx, TOP, FALSE, TRUE);
      }
      else if (key == 'b')
      {
         moveCDKTemplate (template, template->win->_begx, BOTTOM, FALSE, TRUE);
      }
      else if (key == 'l')
      {
         moveCDKTemplate (template, LEFT, template->win->_begy, FALSE, TRUE);
      }
      else if (key == 'r')
      {
         moveCDKTemplate (template, RIGHT, template->win->_begy, FALSE, TRUE);
      }
      else if (key == 'c')
      {
         moveCDKTemplate (template, CENTER, template->win->_begy, FALSE, TRUE);
      }
      else if (key == 'C')
      {
         moveCDKTemplate (template, template->win->_begx, CENTER, FALSE, TRUE);
      }
      else if (key == CDK_REFRESH)
      {
         eraseCDKScreen (template->screen);
         refreshCDKScreen (template->screen);
      }
      else if (key == KEY_ESC)
      {
         moveCDKTemplate (template, origX, origY, FALSE, TRUE);
      }
      else if ((key != KEY_RETURN) && (key != KEY_ENTER))
      {
         Beep();
      }
   }
}

/*
 * This function draws the tmeplate widget.
 */
void drawCDKTemplate (CDKTEMPLATE *template, boolean Box)
{
   int x;

   /* Do we need to draw the shadow. */
   if (template->shadowWin != (WINDOW *)NULL)
   {
      drawShadow (template->shadowWin);
   }

   /* Box it if needed */
   if (Box)
   {
      attrbox (template->win,
		template->ULChar, template->URChar,
		template->LLChar, template->LRChar,
		template->HChar,  template->VChar,
		template->BoxAttrib);
   }

   /* Draw in the title if there is one. */
   if (template->titleLines != 0)
   {
      for (x=0; x < template->titleLines; x++)
      {
         writeChtype (template->win,
			template->titlePos[x],
			x+1,
			template->title[x],
			HORIZONTAL, 0,
			template->titleLen[x]);
      }
   }
   touchwin (template->win);
   wrefresh (template->win);
   
   /* Draw the template field. */
   drawCDKTemplateField (template);
}

/*
 * This draws the template field.
 */
void drawCDKTemplateField (CDKTEMPLATE *template)
{
   /* Declare local variables. */
   chtype fieldColor	= (chtype)NULL;
   int infolen		= (int)strlen (template->info);
   int x		= 0;

   /* Draw in the label and the template object. */
   if (template->labelWin != (WINDOW *)NULL)
   {
      writeChtype (template->labelWin, 0, 0, 
			template->label,
			HORIZONTAL, 0,
			template->labelLen);
      touchwin (template->labelWin);
      wrefresh (template->labelWin);
   }

   /* Draw in the template... */
   if (template->overlay != (chtype *)NULL)
   {
      writeChtype (template->fieldWin, 0, 0,
			template->overlay,
			HORIZONTAL, 0,
			template->overlayLen);
   }

   /* Adjust the cursor. */
   if (infolen != 0)
   {
      int pos	= 0;
      for (x=0; x < template->fieldWidth; x++)
      {
         if ((template->plate[x] == '#' ||
		template->plate[x] == 'A' ||
		template->plate[x] == 'C' ||
		template->plate[x] == 'c' ||
		template->plate[x] == 'M' ||
		template->plate[x] == 'X' ||
		template->plate[x] == 'x') &&
		pos < infolen)
         {
            fieldColor = template->overlay[x] & A_ATTRIBUTES;
            mvwaddch (template->fieldWin, 0, x, template->info[pos++]|fieldColor);
         }
      }
      wmove (template->fieldWin, 0, template->screenPos);
   }
   else
   {
      adjustCDKTemplateCursor (template, +1);
   }
   touchwin (template->fieldWin);
   wrefresh (template->fieldWin);
}

/*
 * This function adjusts the cursor for the template.
 */
void adjustCDKTemplateCursor (CDKTEMPLATE *template, int direction)
{
   while (template->plate[template->platePos] != '#' &&
		template->plate[template->platePos] != 'A' &&
		template->plate[template->platePos] != 'C' &&
		template->plate[template->platePos] != 'c' &&
		template->plate[template->platePos] != 'M' &&
		template->plate[template->platePos] != 'X' &&
		template->plate[template->platePos] != 'x'  &&
		template->platePos < template->fieldWidth)
   {
      template->platePos += direction;
      template->screenPos += direction;
   }
   wmove (template->fieldWin, 0, template->screenPos);
   wrefresh (template->fieldWin);
}

/*
 * These functions set the drawing characters of the widget.
 */
void setCDKTemplateULChar (CDKTEMPLATE *template, chtype character)
{
   template->ULChar = character;
}
void setCDKTemplateURChar (CDKTEMPLATE *template, chtype character)
{
   template->URChar = character;
}
void setCDKTemplateLLChar (CDKTEMPLATE *template, chtype character)
{
   template->LLChar = character;
}
void setCDKTemplateLRChar (CDKTEMPLATE *template, chtype character)
{
   template->LRChar = character;
}
void setCDKTemplateVerticalChar (CDKTEMPLATE *template, chtype character)
{
   template->VChar = character;
}
void setCDKTemplateHorizontalChar (CDKTEMPLATE *template, chtype character)
{
   template->HChar = character;
}
void setCDKTemplateBoxAttribute (CDKTEMPLATE *template, chtype character)
{
   template->BoxAttrib = character;
}

/*
 * This sets the background color of the widget.
 */ 
void setCDKTemplateBackgroundColor (CDKTEMPLATE *template, char *color)
{
   chtype *holder = (chtype *)NULL;
   int junk1, junk2;

   /* Make sure the color isn't NULL. */
   if (color == (char *)NULL)
   {
      return;
   }

   /* Convert the value of the environment variable to a chtype. */
   holder = char2Chtype (color, &junk1, &junk2);

   /* Set the widgets background color. */
   wbkgd (template->win, holder[0]);
   wbkgd (template->fieldWin, holder[0]);
   if (template->labelWin != (WINDOW *)NULL)
   {
      wbkgd (template->labelWin, holder[0]);
   }

   /* Clean up. */
   freeChtype (holder);
}

/*
 * This function destroys this widget.
 */
void destroyCDKTemplate (CDKTEMPLATE *template)
{
   int x;

   /* Erase the object. */
   eraseCDKTemplate (template);

   /* Clear out the character pointers. */
   freeChtype (template->label);
   freeChtype (template->overlay);
   freeChar (template->plate);
   freeChar (template->info);
   for (x=0; x < template->titleLines; x++)
   {
      freeChtype (template->title[x]);
   }

   /* Delete the windows. */
   deleteCursesWindow (template->fieldWin);
   deleteCursesWindow (template->labelWin);
   deleteCursesWindow (template->shadowWin);
   deleteCursesWindow (template->win);

   /* Unregister this object. */
   unregisterCDKObject (vTEMPLATE, template);

   /* Finish cleaning up. */
   free (template);
}

/*
 * This function erases the widget.
 */
void eraseCDKTemplate (CDKTEMPLATE *template)
{
   eraseCursesWindow (template->fieldWin);
   eraseCursesWindow (template->labelWin);
   eraseCursesWindow (template->win);
   eraseCursesWindow (template->shadowWin);
}

/*
 * This function sets the value given to the template.
 */
void setCDKTemplate (CDKTEMPLATE *template, char *newValue, boolean Box)
{
   setCDKTemplateValue (template, newValue);
   setCDKTemplateBox (template, Box);
}

/*
 * This function sets the value given to the template.
 */
void setCDKTemplateValue (CDKTEMPLATE *template, char *newValue)
{
   /* Declare local variables. */
   int len		= 0;
   int copychars	= 0;
   int x;

   /* Just to be sure, if lets make sure the new value isn't NULL. */
   if (newValue == (char *)NULL)
   { 
      /* Then we want to just erase the old value. */
      cleanChar (template->info, template->fieldWidth, '\0');

      /* Set the cursor place values. */
      template->screenPos = 0;
      template->platePos = 0;
      template->infoPos = 0;
      return;
   }

   /* Determine how many characters we need to copy. */
   len		= (int)strlen (newValue);
   copychars	= MINIMUM (len, template->fieldWidth);

   /* OK, erase the old value, and copy in the new value. */
   cleanChar (template->info, template->fieldWidth, '\0');
   strncpy (template->info, newValue, copychars);

   /* Use the function which handles the input of the characters. */
   for (x=0; x < len; x++)
   {
      ((TEMPLATECB)template->callbackfn)(template, (chtype)newValue[x]);
   }
}
char *getCDKTemplateValue (CDKTEMPLATE *template)
{
   return template->info;
}

/*
 * This sets the minimum number of characters to enter into the widget.
 */
void setCDKTemplateMin (CDKTEMPLATE *template, int min)
{
   if (min >= 0)
   {
      template->min = min;
   }
}
int getCDKTemplateMin (CDKTEMPLATE *template)
{
   return template->min;
}

/*
 * This sets the box attribute of the template widget.
 */
void setCDKTemplateBox (CDKTEMPLATE *template, boolean Box)
{
   template->box = Box;
}
boolean getCDKTemplateBox (CDKTEMPLATE *template)
{
   return template->box;
}

/*
 * This erases the information in the template widget.
 */
void cleanCDKTemplate (CDKTEMPLATE *template)
{
   cleanChar (template->info, template->fieldWidth, '\0');
}

/*
 * This function sets the callback function for the widget.
 */
void setCDKTemplateCB (CDKTEMPLATE *template, TEMPLATECB callback)
{
   template->callbackfn = (void *)callback;
}

/*
 * This function sets the pre-process function.
 */
void setCDKTemplatePreProcess (CDKTEMPLATE *template, PROCESSFN callback, void *data)
{
   template->preProcessFunction = callback;
   template->preProcessData = data;
}
 
/*
 * This function sets the post-process function.
 */
void setCDKTemplatePostProcess (CDKTEMPLATE *template, PROCESSFN callback, void *data)
{
   template->postProcessFunction = callback;
   template->postProcessData = data;
}
